@@ -6,6 +6,7 @@ import hashlib
 import itertools
 import logging
 
+from pytype import datatypes
 from pytype.abstract import _classes
 from pytype.abstract import _function_base
 from pytype.abstract import _instance_base
@@ -236,6 +237,7 @@ class SignedFunction(_function_base.Function):
     return subst
 
   def _match_args_sequentially(self, node, args, alias_map, match_all_views):
+    substs = None
     for name, arg, formal in self.signature.iter_args(args):
       if formal is None:
         continue
@@ -243,13 +245,16 @@ class SignedFunction(_function_base.Function):
         # The annotation is Tuple or Dict, but the passed arg only has to be
         # Iterable or Mapping.
         formal = self.ctx.convert.widen_type(formal)
-      match_result = self.ctx.matcher(node).bad_matches(arg, formal)
+      match_result = self.ctx.matcher(node).bad_matches(arg, formal, name)
       if not function.match_succeeded(match_result, match_all_views, self.ctx):
-        bad_arg = function.BadParam(
-            name=name, expected=formal, error_details=match_result[0][0][1])
-        raise function.WrongArgTypes(
-            self.signature, args, self.ctx, bad_param=bad_arg)
-    return [{}]
+        raise function.WrongArgTypes(self.signature, args, self.ctx,
+                                     bad_param=match_result[0][0].expected)
+      if any(match_result[1]):
+        assert substs is None
+        substs = match_result[1]
+    if not substs:
+      substs = [datatypes.HashableDict()]
+    return substs
 
   def get_first_opcode(self):
     return None
