@@ -1,6 +1,7 @@
 """Constructs related to type annotations."""
 
 import collections
+import dataclasses
 import logging
 import typing
 from typing import Mapping, Tuple, Type, Union as _Union
@@ -290,19 +291,22 @@ class AnnotationContainer(AnnotationClass):
           actual = param_value.instantiate(root_node)
         bad, _ = self.ctx.matcher(root_node).bad_matches(actual, formal_param)
         if bad:
-          if not isinstance(param_value, TypeParameter):
-            # If param_value is not a TypeVar, we substitute in TypeVar bounds
-            # and constraints in formal_param for a more helpful error message.
-            formal_param = self.ctx.annotation_utils.sub_one_annotation(
-                root_node, formal_param, [{}])
-            details = None
-          elif isinstance(formal_param, TypeParameter):
-            details = (f"TypeVars {formal_param.name} and {param_value.name} "
-                       "have incompatible bounds or constraints.")
+          if isinstance(param_value, TypeParameter):
+            # bad_matches replaces type parameters in the expected type with
+            # their concrete values, which is usually what we want. But when the
+            # actual type is a type parameter, then it's more helpful to show
+            # the expected type as a type parameter as well.
+            bad = [dataclasses.replace(match, expected=dataclasses.replace(
+                match.expected, typ=formal_param)) for match in bad]
+            if isinstance(formal_param, TypeParameter):
+              details = (f"TypeVars {formal_param.name} and {param_value.name} "
+                         "have incompatible bounds or constraints.")
+            else:
+              details = None
           else:
             details = None
           self.ctx.errorlog.bad_concrete_type(
-              self.ctx.vm.frames, root_node, formal_param, actual, bad, details)
+              self.ctx.vm.frames, root_node, bad, details)
           return self.ctx.convert.unsolvable
 
     try:
