@@ -4,7 +4,7 @@ import collections
 import dataclasses
 import itertools
 import logging
-from typing import Any, Dict, Optional, Sequence, Tuple
+from typing import Dict, Optional, Sequence, Tuple
 
 import attrs
 
@@ -39,6 +39,8 @@ def get_signatures(func):
     return get_signatures(func.method)
   elif _isinstance(func, "SignedFunction"):
     return [func.signature]
+  elif _isinstance(func, "AMBIGUOUS_OR_EMPTY"):
+    return [Signature.from_any()]
   elif func.__class__.__name__ == "PropertyInstance":
     # NOTE: We typically do not want to treat a PropertyInstance as a callable.
     # This check is here due to a crash in the matcher when applying a method
@@ -261,6 +263,20 @@ class Signature:
         annotations={},
     )
 
+  @classmethod
+  def from_any(cls):
+    """Treat `Any` as `f(...) -> Any`."""
+    return cls(
+        name="<callable>",
+        param_names=(),
+        posonly_count=0,
+        varargs_name="args",
+        kwonly_params=(),
+        kwargs_name="kwargs",
+        defaults={},
+        annotations={},
+    )
+
   def has_param(self, name):
     return name in self.param_names or name in self.kwonly_params or (
         name == self.varargs_name or name == self.kwargs_name)
@@ -663,19 +679,11 @@ class DictKeyMissing(Exception, ReturnValueMixin):
     return not self.__gt__(other)
 
 
-@dataclasses.dataclass(eq=True, frozen=True)
-class BadParam:
-  name: str
-  expected: _base.BaseValue
-  # Should be matcher.ErrorDetails but can't use due to circular dep.
-  error_details: Optional[Any] = None
-
-
 @dataclasses.dataclass(eq=True, frozen=True)
 class BadCall:
   sig: Signature
   passed_args: Sequence[Tuple[str, _base.BaseValue]]
-  bad_param: Optional[BadParam]
+  bad_param: Optional[abstract_utils.BadType]
 
 
 class InvalidParameters(FailedFunctionCall):
@@ -913,12 +921,12 @@ def match_all_args(ctx, node, func, args):
 
 
 def match_succeeded(match_result, match_all_views, ctx):
-  bad_matches, any_match = match_result
+  bad_matches, substs = match_result
   if not bad_matches:
     return True
   if match_all_views or ctx.options.strict_parameter_checks:
     return False
-  return any_match
+  return bool(substs)
 
 
 def has_visible_namedarg(node, args, names):
