@@ -558,8 +558,9 @@ class ErrorLog(ErrorLogBase):
         t.ctx.pytd_convert.OutputMode.DETAILED):
       return self._pytd_print(generic)
 
-  def _print_as_return_types(self, node, formal, actual, bad):
+  def _print_as_return_types(self, node, bad):
     """Print the actual and expected values for a return type."""
+    formal = bad[0].expected.typ
     convert = formal.ctx.pytd_convert
     with convert.set_output_mode(convert.OutputMode.DETAILED):
       expected = self._pytd_print(formal.get_instance_type(node))
@@ -571,7 +572,9 @@ class ErrorLog(ErrorLogBase):
       output_mode = convert.OutputMode.DETAILED
     with convert.set_output_mode(output_mode):
       bad_actual = self._pytd_print(pytd_utils.JoinTypes(
-          view[actual].data.to_type(node, view=view) for view, _ in bad))
+          match.actual_binding.data.to_type(node, view=match.view)
+          for match in bad))
+      actual = bad[0].actual
       if len(actual.bindings) > len(bad):
         full_actual = self._pytd_print(pytd_utils.JoinTypes(
             v.to_type(node) for v in actual.data))
@@ -582,16 +585,15 @@ class ErrorLog(ErrorLogBase):
     error_details = self._prepare_errorlog_details(bad)
     return (fmt(expected), fmt(bad_actual), fmt(full_actual), error_details)
 
-  def _print_as_function_def(self, fn):
-    assert isinstance(fn, abstract.Function)
+  def _print_as_function_def(self, fn: abstract.Function) -> str:
     convert = fn.ctx.pytd_convert
     name = fn.name.rsplit(".", 1)[-1]  # We want `def bar()` not `def Foo.bar()`
     with convert.set_output_mode(convert.OutputMode.DETAILED):
       pytd_def = convert.value_to_pytd_def(fn.ctx.root_node, fn, name)
     return pytd_utils.Print(pytd_def)
 
-  def _print_protocol_error(self, error):
-    """Pretty-print the matcher.ProtocolError instance."""
+  def _print_protocol_error(self, error: matcher.ProtocolError) -> str:
+    """Pretty-print the protocol error."""
     convert = error.left_type.ctx.pytd_convert
     with convert.set_output_mode(convert.OutputMode.DETAILED):
       left = self._pytd_print(error.left_type.get_instance_type())
@@ -636,10 +638,10 @@ class ErrorLog(ErrorLogBase):
       ret += "\nTypedDict extra keys: " + ", ".join(error.extra)
     if error.bad:
       ret += "\nTypedDict type errors: "
-      for k, v, typ, bad in error.bad:
-        for view, _ in bad:
-          actual = self._print_as_actual_type(view[v].data)
-          expected = self._print_as_expected_type(typ)
+      for k, bad in error.bad:
+        for match in bad:
+          actual = self._print_as_actual_type(match.actual_binding.data)
+          expected = self._print_as_expected_type(match.expected.typ)
           ret += f"\n  {{'{k}': ...}}: expected {expected}, got {actual}"
     return ret
 
@@ -654,8 +656,8 @@ class ErrorLog(ErrorLogBase):
   def _prepare_errorlog_details(self, bad):
     """Prepare printable annotation matching errors."""
     details = collections.defaultdict(set)
-    for _, err in bad:
-      d = self._print_error_details(err)
+    for match in bad:
+      d = self._print_error_details(match.error_details)
       for i, detail in enumerate(d):
         if detail:
           details[i].add(detail)
@@ -715,7 +717,7 @@ class ErrorLog(ErrorLogBase):
     for prefix, name in self._iter_sig(sig):
       suffix = " = ..." if name in sig.defaults else ""
       if bad_param and name == bad_param.name:
-        type_str = self._print_as_expected_type(bad_param.expected)
+        type_str = self._print_as_expected_type(bad_param.typ)
         suffix = ": " + type_str + suffix
       yield prefix, name, suffix
 
@@ -871,7 +873,7 @@ class ErrorLog(ErrorLogBase):
       operator, left_operand, right_operand = operation
       operator_name = _function_name(operator, capitalize=True)
       expected_right_operand = self._print_as_expected_type(
-          bad_call.bad_param.expected)
+          bad_call.bad_param.typ)
       details = (f"{operator_name} on {left_operand} expects "
                  f"{expected_right_operand}")
       self._unsupported_operands(
@@ -906,14 +908,14 @@ class ErrorLog(ErrorLogBase):
     self._invalid_parameters(stack, message, bad_call)
 
   @_error_name("not-callable")
-  def not_callable(self, stack, func):
+  def not_callable(self, stack, func, details=None):
     """Calling an object that isn't callable."""
     if isinstance(func, abstract.InterpreterFunction) and func.is_overload:
       prefix = "@typing.overload-decorated "
     else:
       prefix = ""
     message = f"{prefix}{func.name!r} object is not callable"
-    self.error(stack, message, keyword=func.name)
+    self.error(stack, message, keyword=func.name, details=details)
 
   @_error_name("not-indexable")
   def not_indexable(self, stack, name, generic_warning=False):
@@ -993,10 +995,10 @@ class ErrorLog(ErrorLogBase):
                details=details, keyword=base_cls)
 
   @_error_name("bad-return-type")
-  def bad_return_type(self, stack, node, formal, actual, bad):
+  def bad_return_type(self, stack, node, bad):
     """Logs a [bad-return-type] error."""
     expected, bad_actual, full_actual, error_details = (
-        self._print_as_return_types(node, formal, actual, bad))
+        self._print_as_return_types(node, bad))
     if full_actual == bad_actual:
       message = "bad return type"
     else:
@@ -1018,9 +1020,8 @@ class ErrorLog(ErrorLogBase):
     self.error(stack, message, details)
 
   @_error_name("bad-concrete-type")
-  def bad_concrete_type(self, stack, node, formal, actual, bad, details=None):
-    expected, actual, _, error_details = (
-        self._print_as_return_types(node, formal, actual, bad))
+  def bad_concrete_type(self, stack, node, bad, details=None):
+    expected, actual, _, error_details = self._print_as_return_types(node, bad)
     full_details = ["       Expected: ", expected, "\n",
                     "Actually passed: ", actual]
     if details:
