@@ -11,10 +11,10 @@ from pytype import config
 from pytype import file_utils
 from pytype import load_pytd
 from pytype import module_utils
+from pytype.imports import pickle_utils
 from pytype.platform_utils import path_utils
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
-from pytype.pytd import serialize_ast
 from pytype.pytd import visitors
 from pytype.tests import test_base
 
@@ -723,7 +723,7 @@ class PickledPyiLoaderTest(test_base.UnitTest):
 
   def _pickle_modules(self, loader, tempdir, *modules):
     for module in modules:
-      serialize_ast.StoreAst(
+      pickle_utils.StoreAst(
           loader._modules[module.module_name].ast,
           self._get_path(tempdir, module.file_name + ".pickled"))
 
@@ -741,7 +741,7 @@ class PickledPyiLoaderTest(test_base.UnitTest):
       loader, ast = self._load_ast(tempdir=d, module=module1)
       self._pickle_modules(loader, d, module1, module2)
       pickled_ast_filename = self._get_path(d, module1.file_name + ".pickled")
-      result = serialize_ast.StoreAst(ast, pickled_ast_filename)
+      result = pickle_utils.StoreAst(ast, pickled_ast_filename)
       self.assertIsNone(result)
 
       loaded_ast = self._load_pickled_module(d, module1)
