@@ -1,12 +1,11 @@
-# Lint as: python3
 """Test errors.py."""
 
+import io
 import math
-import tempfile
 import time
 
 from pytype import metrics
-from six import moves
+from pytype.platform_utils import tempfile as compatible_tempfile
 
 import unittest
 
@@ -63,21 +62,24 @@ class MetricsTest(unittest.TestCase):
     # Create a counter, increment it, and dump it.
     c1 = metrics.Counter("foo")
     c1.inc(1)
-    dump = metrics.dump([c1], encoding=None)
+    dump = io.StringIO("")
+    metrics.dump_all([c1], dump)
     # Reset metrics, merge from dump, which will create a new metric.
     metrics._prepare_for_test()
     self.assertFalse(metrics._registered_metrics)
-    metrics.merge_from_file(moves.cStringIO(dump))
+    dump.seek(0)
+    metrics.merge_from_file(dump)
     m = metrics._registered_metrics["foo"]
     self.assertEqual(1, m._total)
     # Merge again, this time it will merge data into the existing metric.
-    metrics.merge_from_file(moves.cStringIO(dump))
+    dump.seek(0)
+    metrics.merge_from_file(dump)
     self.assertEqual(2, m._total)
     # It's an error to merge an incompatible type.
     metrics._prepare_for_test()
     _ = metrics.MapCounter("foo")
-    self.assertRaises(TypeError, metrics.merge_from_file,
-                      moves.cStringIO(dump))
+    dump.seek(0)
+    self.assertRaises(TypeError, metrics.merge_from_file, dump)
 
   def test_get_metric(self):
     c1 = metrics.get_metric("foo", metrics.Counter)
@@ -294,13 +296,13 @@ class MetricsContextTest(unittest.TestCase):
     self._counter = metrics.Counter("foo")
 
   def test_enabled(self):
-    with tempfile.NamedTemporaryFile() as out:
+    with compatible_tempfile.NamedTemporaryFile() as out:
       out.close()
       with metrics.MetricsContext(out.name):
         self._counter.inc()
       self.assertEqual(1, self._counter._total)
       with open(out.name) as f:
-        dumped = metrics.load(f)
+        dumped = metrics.load_all(f)
         self.assertEqual(len(dumped), 1)
         self.assertEqual("foo", dumped[0].name)
         self.assertEqual("foo: 1", str(dumped[0]))
