@@ -7,7 +7,6 @@ from typing import Any, Tuple
 from pytype import utils
 from pytype.pytd import pytd
 from pytype.pytd.codegen import pytdgen
-from pytype.pytd.parse import node as pytd_node
 
 # pylint: disable=g-import-not-at-top
 if sys.version_info >= (3, 8):
@@ -77,10 +76,6 @@ class ParseError(Exception):
     return "\n".join(lines)
 
 
-# Type aliases
-Parameters = Tuple[pytd_node.Node, ...]
-
-
 class Ellipsis:  # pylint: disable=redefined-builtin
   pass
 
