@@ -3,6 +3,7 @@ import pickle
 from pytype import config
 from pytype import file_utils
 from pytype import load_pytd
+from pytype.imports import pickle_utils
 from pytype.platform_utils import path_utils
 from pytype.pytd import pytd_utils
 from pytype.pytd import serialize_ast
@@ -19,7 +20,7 @@ class SerializeAstTest(test_base.UnitTest):
       is_package=False, metadata=None):
     if not (ast and loader):
       ast, loader = self._get_ast(temp_dir=temp_dir, module_name=module_name)
-    serialize_ast.StoreAst(
+    pickle_utils.StoreAst(
         ast,
         pickled_ast_filename,
         is_package=is_package,
@@ -59,7 +60,7 @@ class SerializeAstTest(test_base.UnitTest):
     loader = load_pytd.Loader(config.Options.create(
         python_version=self.python_version, pythonpath=temp_dir.path))
     ast = loader.load_file(module_name, pyi_filename)
-    # serialize_ast.StoreAst sorts the ast for determinism, so we should do the
+    # pickle_utils.StoreAst sorts the ast for determinism, so we should do the
     # same to the original ast to do pre- and post-pickling comparisons.
     loader._modules[module_name].ast = ast = ast.Visit(
         visitors.CanonicalOrderingVisitor())
@@ -85,7 +86,7 @@ class SerializeAstTest(test_base.UnitTest):
       pickled_ast_filename = path_utils.join(d.path, "module1.pyi.pickled")
       module_map = self._store_ast(d, module_name, pickled_ast_filename)
       del module_map[module_name]
-      serialized_ast = pytd_utils.LoadPickle(pickled_ast_filename)
+      serialized_ast = pickle_utils.LoadPickle(pickled_ast_filename)
 
       # The sorted makes the testcase more deterministic.
       serialized_ast = serialized_ast.Replace(class_type_nodes=sorted(
@@ -101,7 +102,7 @@ class SerializeAstTest(test_base.UnitTest):
       ast, _ = self._get_ast(temp_dir=d, module_name="foo.bar.module1")
       pickled_ast_filename = path_utils.join(d.path, "module1.pyi.pickled")
 
-      result = serialize_ast.StoreAst(ast, pickled_ast_filename)
+      result = pickle_utils.StoreAst(ast, pickled_ast_filename)
 
       self.assertIsNone(result)
       with open(pickled_ast_filename, "rb") as fi:
@@ -167,7 +168,7 @@ class SerializeAstTest(test_base.UnitTest):
           d, "module1", pickled_ast_filename, ast=ast, loader=loader)
       del module_map["module1"]
 
-      serialized_ast = pytd_utils.LoadPickle(pickled_ast_filename)
+      serialized_ast = pickle_utils.LoadPickle(pickled_ast_filename)
       loaded_ast = serialize_ast.ProcessAst(
           serialized_ast, module_map)
       # Look up the "SomeClass" in "def func(a: SomeClass), then run
@@ -185,7 +186,7 @@ class SerializeAstTest(test_base.UnitTest):
       original_ast = module_map[module_name]
       del module_map[module_name]
       loaded_ast = serialize_ast.ProcessAst(
-          pytd_utils.LoadPickle(pickled_ast_filename),
+          pickle_utils.LoadPickle(pickled_ast_filename),
           module_map)
 
       self.assertTrue(loaded_ast)
@@ -208,7 +209,7 @@ class SerializeAstTest(test_base.UnitTest):
       del module_map[module_name]
 
       loaded_ast = serialize_ast.ProcessAst(
-          pytd_utils.LoadPickle(pickled_ast_filename),
+          pickle_utils.LoadPickle(pickled_ast_filename),
           module_map)
 
       self.assertTrue(loaded_ast)
@@ -226,7 +227,7 @@ class SerializeAstTest(test_base.UnitTest):
 
       with self.assertRaises(serialize_ast.UnrestorableDependencyError):
         serialize_ast.ProcessAst(
-            pytd_utils.LoadPickle(pickled_ast_filename),
+            pickle_utils.LoadPickle(pickled_ast_filename),
             module_map)
 
   def test_unrestorable_dependency_error_without_module_index(self):
@@ -236,7 +237,7 @@ class SerializeAstTest(test_base.UnitTest):
       module_map = self._store_ast(d, module_name, pickled_ast_filename)
       module_map = {}  # Remove module2
 
-      loaded_ast = pytd_utils.LoadPickle(pickled_ast_filename)
+      loaded_ast = pickle_utils.LoadPickle(pickled_ast_filename)
       loaded_ast.modified_class_types = None  # Remove the index
       with self.assertRaises(serialize_ast.UnrestorableDependencyError):
         serialize_ast.ProcessAst(loaded_ast, module_map)
@@ -251,7 +252,7 @@ class SerializeAstTest(test_base.UnitTest):
       del module_map[original_module_name]
 
       new_module_name = "wurstbrot.module2"
-      serializable_ast = pytd_utils.LoadPickle(pickled_ast_filename)
+      serializable_ast = pickle_utils.LoadPickle(pickled_ast_filename)
       serializable_ast = serialize_ast.EnsureAstName(
           serializable_ast, new_module_name, fix=True)
       loaded_ast = serialize_ast.ProcessAst(serializable_ast, module_map)
@@ -271,7 +272,7 @@ class SerializeAstTest(test_base.UnitTest):
 
       module_map = self._store_ast(
           d, original_module_name, pickled_ast_filename, is_package=True)
-      serializable_ast = pytd_utils.LoadPickle(pickled_ast_filename)
+      serializable_ast = pickle_utils.LoadPickle(pickled_ast_filename)
 
       expected_name = "module1"
       # Check that the module had the expected name before.
@@ -285,7 +286,7 @@ class SerializeAstTest(test_base.UnitTest):
     with file_utils.Tempdir() as d:
       foo = d.create_file("foo.pickle")
       module_map = self._store_ast(d, "foo", foo, ast=self._get_ast(d, "foo"))
-      p = pytd_utils.LoadPickle(foo)
+      p = pickle_utils.LoadPickle(foo)
       ast = serialize_ast.ProcessAst(p, module_map)
       f, = (a for a in ast.aliases if a.name == "foo.f")
       signature, = f.type.signatures
@@ -297,7 +298,7 @@ class SerializeAstTest(test_base.UnitTest):
       pickled_ast_filename = path_utils.join(d.path, "module1.pyi.pickled")
       self._store_ast(
           d, module_name, pickled_ast_filename, metadata=["meta", "data"])
-      serialized_ast = pytd_utils.LoadPickle(pickled_ast_filename)
+      serialized_ast = pickle_utils.LoadPickle(pickled_ast_filename)
       self.assertSequenceEqual(serialized_ast.metadata, ["meta", "data"])
 
 if __name__ == "__main__":
