@@ -3,6 +3,7 @@
 import os
 
 from pytype import pytype_source_utils
+from pytype.platform_utils import path_utils
 
 import unittest
 
@@ -12,23 +13,23 @@ class PytypeSourceUtilsTest(unittest.TestCase):
 
   def setUp(self):
     super().setUp()
-    self.root = os.path.dirname(__file__)
+    self.root = path_utils.dirname(__file__)
 
   def test_pytype_source_dir(self):
     self.assertEqual(self.root, pytype_source_utils.pytype_source_dir())
 
   def test_get_full_path(self):
     self.assertEqual(
-        os.path.join(self.root, "foo/bar"),
-        pytype_source_utils.get_full_path("foo/bar"))
+        path_utils.join(self.root, f"foo{os.path.sep}bar"),
+        pytype_source_utils.get_full_path(f"foo{os.path.sep}bar"))
     self.assertEqual(
-        "/foo/bar",
-        pytype_source_utils.get_full_path("/foo/bar"))
+        f"{os.path.sep}foo{os.path.sep}bar",
+        pytype_source_utils.get_full_path(f"{os.path.sep}foo{os.path.sep}bar"))
 
   def test_list_pytype_files(self):
-    l = list(pytype_source_utils.list_pytype_files("stubs/stdlib"))
+    l = list(pytype_source_utils.list_pytype_files(f"stubs{os.path.sep}stdlib"))
     self.assertIn("_ctypes.pytd", l)
-    self.assertIn("collections/__init__.pytd", l)
+    self.assertIn(f"collections{os.path.sep}__init__.pytd", l)
 
 
 if __name__ == "__main__":
