@@ -12,12 +12,13 @@ from pytype import config
 from pytype import file_utils
 from pytype import single
 from pytype import utils
+from pytype.imports import builtin_stubs
+from pytype.imports import pickle_utils
+from pytype.imports import typeshed
 from pytype.platform_utils import path_utils
 from pytype.platform_utils import tempfile as compatible_tempfile
 from pytype.pyi import parser
-from pytype.pytd import builtin_stubs
 from pytype.pytd import pytd_utils
-from pytype.pytd import typeshed
 from pytype.tests import test_base
 
 import unittest
@@ -128,8 +129,8 @@ class PytypeTest(test_base.UnitTest):
     with open(f1, "rb") as pickle1, open(f2, "rb") as pickle2:
       if pickle1.read() == pickle2.read():
         return
-    out1 = pytd_utils.LoadPickle(f1, compress=True)
-    out2 = pytd_utils.LoadPickle(f2, compress=True)
+    out1 = pickle_utils.LoadPickle(f1, compress=True)
+    out2 = pickle_utils.LoadPickle(f2, compress=True)
     raise AssertionError("\n".join(pytd_utils.DiffNamedPickles(out1, out2)))
 
   def assertOutputStateMatches(self, **has_output):
