@@ -5,8 +5,10 @@ import sys
 import types
 
 from pytype import file_utils
+from pytype.platform_utils import path_utils
 from pytype.tools.analyze_project import config
 from pytype.tools.analyze_project import parse_args
+
 import unittest
 
 
@@ -21,7 +23,7 @@ class TestParser(unittest.TestCase):
   def test_parse_filenames(self):
     filenames = ['a.py', 'b.py']
     args = self.parser.parse_args(filenames)
-    self.assertEqual(args.inputs, {os.path.realpath(f) for f in filenames})
+    self.assertEqual(args.inputs, {path_utils.realpath(f) for f in filenames})
 
   def test_parse_no_filename(self):
     args = self.parser.parse_args([])
@@ -38,7 +40,7 @@ class TestParser(unittest.TestCase):
   def test_parse_exclude(self):
     filenames = ['a.py', 'b.py']
     args = self.parser.parse_args(['--exclude'] + filenames)
-    self.assertEqual(args.exclude, {os.path.realpath(f) for f in filenames})
+    self.assertEqual(args.exclude, {path_utils.realpath(f) for f in filenames})
 
   def test_parse_single_exclude(self):
     filenames = ['a.py', 'b/c.py']
@@ -47,7 +49,8 @@ class TestParser(unittest.TestCase):
         d.create_file(f)
       with file_utils.cd(d.path):
         args = self.parser.parse_args(['--exclude=**/*.py'])
-        self.assertEqual(args.exclude, {os.path.realpath(f) for f in filenames})
+        self.assertEqual(args.exclude,
+                         {path_utils.realpath(f) for f in filenames})
 
   def test_parse_exclude_dir(self):
     filenames = ['foo/f1.py', 'foo/f2.py']
@@ -56,7 +59,8 @@ class TestParser(unittest.TestCase):
         d.create_file(f)
       with file_utils.cd(d.path):
         args = self.parser.parse_args(['--exclude=foo/'])
-        self.assertEqual(args.exclude, {os.path.realpath(f) for f in filenames})
+        self.assertEqual(args.exclude,
+                         {path_utils.realpath(f) for f in filenames})
 
   def test_parse_bad_exclude(self):
     args = self.parser.parse_args(['-x', 'this_file_should_not_exist'])
@@ -97,10 +101,12 @@ class TestParser(unittest.TestCase):
                      f'{sys.version_info.major}.{sys.version_info.minor}')
 
   def test_output(self):
-    self.assertEqual(self.parser.parse_args(
-        ['-o', 'pyi']).output, os.path.join(os.getcwd(), 'pyi'))
-    self.assertEqual(self.parser.parse_args(
-        ['--output', 'pyi']).output, os.path.join(os.getcwd(), 'pyi'))
+    self.assertEqual(
+        self.parser.parse_args(['-o', 'pyi']).output,
+        path_utils.join(path_utils.getcwd(), 'pyi'))
+    self.assertEqual(
+        self.parser.parse_args(['--output', 'pyi']).output,
+        path_utils.join(path_utils.getcwd(), 'pyi'))
 
   def test_no_cache(self):
     self.assertFalse(self.parser.parse_args([]).no_cache)
@@ -110,11 +116,13 @@ class TestParser(unittest.TestCase):
       self.parser.parse_args(['--output', 'pyi', '--no-cache'])
 
   def test_pythonpath(self):
-    d = os.getcwd()
-    self.assertSequenceEqual(self.parser.parse_args(
-        ['-P', ':foo']).pythonpath, [d, os.path.join(d, 'foo')])
-    self.assertSequenceEqual(self.parser.parse_args(
-        ['--pythonpath', ':foo']).pythonpath, [d, os.path.join(d, 'foo')])
+    d = path_utils.getcwd()
+    self.assertSequenceEqual(
+        self.parser.parse_args(['-P', f'{os.pathsep}foo']).pythonpath,
+        [d, path_utils.join(d, 'foo')])
+    self.assertSequenceEqual(
+        self.parser.parse_args(['--pythonpath', f'{os.pathsep}foo']).pythonpath,
+        [d, path_utils.join(d, 'foo')])
 
   def test_keep_going(self):
     self.assertTrue(self.parser.parse_args(['-k']).keep_going)
@@ -134,7 +142,8 @@ class TestParser(unittest.TestCase):
   def test_config_file(self):
     conf = self.parser.config_from_defaults()
     # Spot check a pytype-all arg.
-    self.assertEqual(conf.output, os.path.join(os.getcwd(), '.pytype'))
+    self.assertEqual(conf.output, path_utils.join(path_utils.getcwd(),
+                                                  '.pytype'))
     # And a pytype-single arg.
     self.assertIsInstance(conf.disable, list)
     self.assertFalse(conf.disable)
