@@ -4,7 +4,9 @@ import collections
 import dataclasses
 import re
 import sys
-from typing import Any, List, Optional
+import textwrap
+import types
+from typing import Any, List, Optional, Type, TypeVar
 
 from pytype import analyze
 from pytype import config
@@ -47,6 +49,8 @@ IMPORT_FILE_MARKER = "<__FILE__>"
 # Marker to capture a pending return value while traversing an AST
 _RETURNING_NAME = "RETURNING NAME"
 
+_T = TypeVar("_T")
+
 
 def qualified_method(data):
   """Fully qualify a method call with its class scope."""
@@ -62,6 +66,17 @@ def get_location(node):
   return source.Location(node.lineno, node.col_offset)
 
 
+def get_end_location(node):
+  # These attributes are new in Python 3.8.
+  if sys.version_info[:2] >= (3, 8):
+    end_lineno = node.end_lineno
+    end_col_offset = node.end_col_offset
+  else:
+    end_lineno = node.lineno
+    end_col_offset = node.col_offset
+  return source.Location(end_lineno, end_col_offset)
+
+
 def match_opcodes(opcode_traces, lineno, op_match_list):
   """Get all opcodes matching op_match_list on a given line.
 
@@ -179,7 +194,7 @@ class DocString:
   length: int
 
   @classmethod
-  def from_node(cls, ast, node):
+  def from_node(cls: Type[_T], ast: types.ModuleType, node) -> Optional[_T]:
     """If the first element in node.body is a string, create a docstring."""
 
     # This should only be called on ClassDef and FunctionDef
@@ -190,6 +205,10 @@ class DocString:
       doc_node = node.body[0]
       doc = doc_node.value.s
       length = len(doc)  # we want to preserve the byte length
+      # strip indentation from multiline docstrings
+      if "\n" in doc:
+        first, rest = doc.split("\n", 1)
+        doc = first + "\n" + textwrap.dedent(rest).rstrip()
       return cls(doc, get_location(doc_node), length)
     return None
 
@@ -361,6 +380,7 @@ class Funcall:
   scope: str
   func: str
   location: source.Location
+  end_location: source.Location
   args: List[Any]
   return_type: str
 
@@ -544,6 +564,9 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
     self.classmap = {}
     self.calls = []
     self.function_params = []
+    # Record childof relationships for nested functions/classes
+    self.childof = []
+    self.scope_defn = {}
 
   def _get_location(self, node, args):
     """Get a more accurate node location."""
@@ -652,8 +675,10 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
     return self.add_local_ref(node, **kwargs)
 
   def add_call(self, node, name, func, arg_varnames, return_type):
+    start = get_location(node)
+    end = get_end_location(node)
     self.calls.append(
-        Funcall(name, self.scope_id(), func, get_location(node), arg_varnames,
+        Funcall(name, self.scope_id(), func, start, end, arg_varnames,
                 return_type))
 
   def add_attr(self, node):
@@ -669,6 +694,12 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
         return True
     return False
 
+  def _record_childof(self, node, defn):
+    """Record a childof relationship for nested definitions."""
+    parent = self.scope_defn.get(self.scope_id())
+    if parent:
+      self.childof.append((defn, parent))
+
   def enter_ClassDef(self, node):
     class_name = node_utils.get_name(node, self._ast)
     last_line = max(node.lineno, node.body[0].lineno - 1)
@@ -696,8 +727,10 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
         class_name, node.lineno)
     defn = self.add_local_def(node, data=data,
                               doc=DocString.from_node(self._ast, node))
+    self._record_childof(node, defn)
     self.classmap[d[0]] = defn
     super().enter_ClassDef(node)
+    self.scope_defn[self.scope_id()] = defn
 
   def enter_FunctionDef(self, node):
     last_line = max(node.lineno, node.body[0].lineno - 1)
@@ -713,7 +746,9 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
       data = None
     fn_def = self.add_local_def(node, data=data,
                                 doc=DocString.from_node(self._ast, node))
+    self._record_childof(node, fn_def)
     env = self.add_scope(node)
+    self.scope_defn[self.scope_id()] = fn_def
     # TODO(mdemello): Get pytype data for params
     params = [self.add_local_def(v) for v in node.args.args]
     for i, param in enumerate(params):
@@ -926,6 +961,7 @@ class Indexer:
     self.links = []
     self.function_params = None
     self.function_map = None
+    self.childof = []
 
     # Optionally preserve the pytype vm so we can access the types later
     self.vm = None
@@ -945,6 +981,7 @@ class Indexer:
     self.classmap = v.classmap
     self.calls = v.calls
     self.function_params = v.function_params
+    self.childof = v.childof
 
   def get_def_offsets(self, defloc):
     """Get the byte offsets for a definition."""
