@@ -154,6 +154,12 @@ class IndexerTest(test_base.BaseTest, IndexerTestMixin):
           # x.y as references to remote files
           ("x", ":module:", "x/__init__.py"),
           ("y", ":module:", "x/y.py"),
+          # calls
+          ("X()", "module.X", "p/q.py"),
+          ("X()", "module.X", "f.py"),
+          ("X()", "module.X", "a/b.py"),
+          ("X()", "module.X", "x/y.py"),
+          ("X()", "module.X", "u/v.py"),
       }
 
       f = file_utils.replace_separator
@@ -268,6 +274,19 @@ class IndexerTest(test_base.BaseTest, IndexerTestMixin):
     for r in ix.refs:
       self.assertIsNone(r.data)
 
+  def test_docstring(self):
+    ix = self.index_code("""
+        def f():
+          '''Multiline docstring
+
+          foo
+            bar
+          '''
+    """)
+    d = ix.defs["module.f"]
+    self.assertEqual(d.doc.text, "Multiline docstring\n\nfoo\n  bar")
+    self.assertGreater(d.doc.length, len(d.doc.text))
+
 
 class IndexerTestPy3(test_base.BaseTest, IndexerTestMixin):
 
