@@ -256,6 +256,15 @@ def _process_links(kythe: Kythe, index: indexer.Indexer):
     kythe.add_edge(source=vname, target=target, edge_name=edge_name)
 
 
+def _process_childof(kythe: Kythe, index: indexer.Indexer):
+  """Generate kythe edges for childof relationships."""
+
+  for child, parent in index.childof:
+    source = _make_defn_vname(kythe, index, child)
+    target = _make_defn_vname(kythe, index, parent)
+    kythe.add_edge(source=source, target=target, edge_name="childof")
+
+
 def _process_calls(kythe, index):
   """Generate kythe edges for function calls."""
 
@@ -278,8 +287,9 @@ def _process_calls(kythe, index):
     if call_defn:
       target = _make_defn_vname(kythe, index, call_defn)
       if target:
-        start, end = index.get_ref_bounds(call_ref)
-        anchor_vname = kythe.anchor_vname(start, end)
+        start, _ = index.get_ref_bounds(call_ref)
+        end = index.source.get_offset(call.end_location)
+        anchor_vname = kythe.add_anchor(start, end)
         kythe.add_edge(source=anchor_vname, target=target, edge_name="ref/call")
         # The call is a child of the enclosing function/class (this lets us
         # generate call graphs).
@@ -300,5 +310,6 @@ def generate_graph(index, kythe_args):
   _process_deflocs(kythe, index)
   _process_params(kythe, index)
   _process_links(kythe, index)
+  _process_childof(kythe, index)
   _process_calls(kythe, index)
   return kythe
